#!/bin/bash
#
# Runs multiple realizations of a set of experiments (defined in
# the config.yml)
#
# How to use this script:
#
# 1) clone the autosubmit-api repo, github.com:BSC-ES/autosubmit-api.git
# 2) install autosubmit in a virtual environment  (using venv)
# 3) update paths that are hardcoded in the configuration variable part
#
# (nohup) ./run_multiple_realizations.sh hyperqueue 30
# to run 30 times the testing suite of hyperqueue
#
# This script will create status for each realization so that the
# user is aware of how the experiments finished.
#
# Copyright (c) 2025  Manuel G. Marciani
# BSC-CNS - Earth Sciences
#

# CONFIGURATION VARIABLES 
# where the mamba environments are intalled
MAMBA_ROOT_PREFIX=${HOME}/Documents/local/dev/venvs
# where micromamba is installed
MICROMAMBA_PATH=${HOME}/Documents/local/dev/micromamba
# where the mamba environment of the testing_suite is installed
TESTING_SUITE_PATH=/home/mgimenez/Documents/local/dev/venvs/testing_suite
# where the venv of autosubmit is located
AUTOSUBMIT_VENV=/home/mgimenez/Documents/local/dev/venvs/autosubmit3
# folder where the autosubmit database is located
AUTOSUBMIT_DATABASE=/home/mgimenez/autosubmit
# port which the api will listen 
AS_API_PORT=8099 

# loads the testing suite environment from t
function load_testing_suite() {
    # load the testing_suite environment
    echo -n "testing_suite_wrapper.sh: Info: Loading micromamba..."    
    export PATH=${MICROMAMBA_PATH}/:$PATH
    if [[ -n "$(micromamba 2>&1 /dev/null)" ]]; then 
        echo "Loaded!"
    else
        echo ""
        echo "testing_suite_wrapper.sh: Error: Could not load Micromamba from $MICROMAMBA_PATH"    
    fi
    eval "$(micromamba shell hook --shell zsh)"

    echo -n "testing_suite_wrapper.sh: Info: Loading the Testing suite..."    
    
    micromamba activate $TESTING_SUITE_PATH 

    if [[ -n "$(testing_suite 2>&1 /dev/null)" ]]; then 
        echo "Loaded!"
    else
        echo ""
        echo "testing_suite_wrapper.sh: Error: Could not load the testing_suite from $TESTING_SUITE_PATH"    
    fi

}

function load_autosubmit_api() {
    # start the api docker image
    echo -n "testing_suite_wrapper.sh: Info: Starting the API listening at $AS_API_PORT with databases at $AUTOSUBMIT_DATABASE.."
    # check if the container is already running    
    if [[ -z "$(docker container inspect autosubmit-api-container 2> /dev/null | grep running)" ]]; then 
        docker run --name autosubmit-api-container \
            --rm -d -p $AS_API_PORT:8000 \
            --mount type=bind,src=$AUTOSUBMIT_DATABASE,dst=/app/autosubmit/database,bind-propagation=shared \
            --mount type=bind,src=$AUTOSUBMIT_DATABASE,dst=/app/autosubmit/experiments,bind-propagation=shared \
            -e "PROTECTION_LEVEL=NONE" \
            autosubmit-api > /dev/null
        # sleep until the api responds 
        while [[ -z "$(curl -f localhost:$AS_API_PORT 2> /dev/null | grep Autosubmit)" ]]; do
            sleep 0.2
            echo -n "."
        done 
        echo "API started!"    
    else
        echo "API already running!"    
    fi
}

function stop_autosubmit_api() {
    echo -n "testing_suite_wrapper.sh: Info: Stopping api..."    
    if [[ "$(docker kill autosubmit-api-container 2>&1)" == "autosubmit-api-container" ]]; then
        echo "Stopped!"    
    else 
        echo ""
        echo "testing_suite_wrapper.sh: Error: could not stop the api"    
    fi
}

function load_autosubmit() {
    echo -n "testing_suite_wrapper.sh: Info: Loading Autosubmit..."    
    # load the autosubmit environment
    source $AUTOSUBMIT_VENV/bin/activate

    if [[ -n "$(autosubmit 2>&1 /dev/null)" ]]; then 
        echo "Loaded!"
    else
        echo "testing_suite_wrapper.sh: Error: Could not load Autosubmit from venv $AUTOSUBMIT_VENV"    
    fi

}

function setup_environment() {
    #
    # This function sets the api, the mamba environment with testing suite, and
    # the autosubmit environment
    #
    
    load_autosubmit
    load_autosubmit_api
    load_testing_suite 

}

function stop_api() {
    stop_autosubmit_api
}

# run locally the testing suite

case "$1" in 
    create)
        setup_environment
        testing_suite --skip_folder_check --create 
        #stop_api
        ;;

    refresh)
        setup_environment
        testing_suite --skip_folder_check --refresh clone
        #stop_api
        ;;
    
    status) # check the status of the jobs
        setup_environment
        testing_suite --skip_folder_check --status
        #stop_api
        ;;
    
    run) # the testing suite 
        if [[ -z "$2" ]]; then
            echo "testing_suite_wrapper.sh: Error: You have to specify the number of interactions"
            exit 
        fi 
        setup_environment
        for (( i=0 ; i < $2; ++i )); do
            echo "testing_suite_wrapper.sh: INFO: Reset experiments (running create) of iteration $i" 
            testing_suite --skip_folder_check --create 
            echo "testing_suite_wrapper.sh: INFO: Running experiments of iteration $i " 
            testing_suite --skip_folder_check --skip_folder_check --run 
            echo "testing_suite_wrapper.sh: INFO: Creating status files of iteration $i" 
            testing_suite --skip_folder_check --status
        done        
        #stop_api
        ;;

    status) # check the status of the jobs
        setup_environment
        testing_suite --status
        #stop_api
    ;;

    set_config_variable) # check the status of the jobs
        if [[ -z "$2" ]]; then
            echo "testing_suite_wrapper.sh: Error: You have to specify the name of the variable to substitute, the new value, and the file that will be altered. ./testing_suite_wrapper.sh VARIABLE_NAME VALUE FILE" 
            exit
        fi
        if [[ -z "$3" ]]; then
            echo "testing_suite_wrapper.sh: Error: You have to specify the new value and the file that will be altered. ./testing_suite_wrapper.sh $2 VALUE FILE" 
            exit
        fi
        if [[ -z "$4" ]]; then
            echo "testing_suite_wrapper.sh: Error: You have to specify the file that will be altered (autosubmit, platform, expdef,  ./testing_suite_wrapper.sh $2 $3 FILE" 
            exit
        fi
        setup_environment
        testing_suite --set_config_variable $2 $3 $4
        #stop_api
    ;;

    "") # just output experiments and description
        setup_environment
        testing_suite
        #stop_api
    ;;
    
    *)
        echo "testing_suite_wrapper.sh: Error: Incorrect command. You have to choose between 'run', 'status', 'refresh', 'set_config_variable', or 'create'"
    ;;
esac 

